% show_trouble.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%clc; 
clear all; close all;

global epsilon
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialize parameters

N = 120;

if 1
    % regulare grid
    x_full = linspace(0,1,N+2)';
    x = x_full(2:end-1);
else
    % random grid
    x = sort(rand(N,1));
    x_full = [0; x; 1];
end
    
epsilon = 0.5; % take 0.5 0.05 0.005 0.0005

% exact solution: for this problem still available
u_ex = find_ex(x_full);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% build the FEM matrix
A = system_matrix(x_full,N);
    
% rhs
F = 0.5*(x_full(3:end)-x_full(1:end-2));


U = A\F;

U = [0; U; 0]; % add the boundary points
    

% cg for the symmetrized system
B = A'*A;
b = A'*F;

maxit = 5000;
toler = 1e-8;

u = zeros(N,1); 
r = b - B*u;

rho = r'*r;

for ctr = 1:maxit
    
    if ctr==1
        pk = r;
    else
        beta = rho/rho_k_minus_2;
        pk = r + beta*pk;
    end
    
    w = B*pk;
    alpha = rho/(pk'*w);
    
    u = u + alpha*pk;    
    r = r - alpha*w;
    
    rho_k_minus_2 = rho;
    rho = r'*r;
    
    
    nr = norm(r);
    res(ctr) = nr;
    
    if nr<=toler
        disp(['converged at iteration ',num2str(ctr)]);
        break;
    end
    
end

% analyse the eigenvalues of the symmetrised system
E = eig(full(B));
disp('max EV')
disp(max(E));
disp('min EV')
disp(min(E));

disp('Condition number')
disp(max(E)/min(E))

plot(x_full,U);